
#####################################################################
#
# data.tcl: reading of xcrvfit data files
#
#####################################################################

#####################################################################
#
# process the dataFile field
#
proc read_data { } {
	global xwin xcv myData panvar

	set iFile 0

# Trim whitespace
	set line [string trim $panvar(data,filename)]
	if { [string length $line] == 0 } {
		return FALSE
	}
	set panvar(data,filename) $line

# what format of input file do we have
	find_file_type $panvar(data,filename)
	u_log "Input data '$panvar(data,filename)' is in format: '$panvar(data,format)'."
	if {[string compare $panvar(data,format) unreadable] == 0} {
		if { $xcv(os) == "AQUA" } {
			u_error "Cannot open data file: '$panvar(data,filename)' For mac osx aqua use full path names or browse button."
		} else {
			u_error "Cannot open data file: '$panvar(data,filename)'"
		}
	}

# convert it to our standard format
	if {[string compare $panvar(data,format) kay] == 0} {
		set prog $xcv(exe,KAY2CRV)
	}
	if {[string compare $panvar(data,format) jfit] == 0} {
		set prog $xcv(exe,JFIT2CRV)
	}
	if {[string compare $panvar(data,format) fp] == 0} {
		set prog $xcv(exe,FP2CRV)
	}

	if {[string compare $panvar(data,format) crvfit] != 0} {
		u_log "Converting data with program: '$prog'."
		set fn [file tail $panvar(data,filename)]
		set panvar(data,crvFile) $fn.crvfit
		set cmd "$prog $panvar(data,filename) $panvar(data,crvFile)"
		if [catch {eval exec $cmd} err] {
			u_error "Problems converting your '$panvar(data,format)' input file '$panvar(data,filename)'. Error = '$err'. Please check your input file format."
		}
		if ![file readable $panvar(data,crvFile)] {
			u_error "Problems converting your '$panvar(data,format)' input file '$panvar(data,filename)'. No output generated. Please check your input file format."
		}
	} else {
		set panvar(data,crvFile) $panvar(data,filename)
	}

	# read the crvfit format data
	read_crvfit_data $panvar(data,crvFile) 0
	return TRUE
}

#####################################################################
proc read_crvfit_data { fn iFile } {
	global myData panvar xcv

	set errMsg ""
	set iSetNum 0
	set iDataNum 0

	set xMin ""
	set newData($iSetNum,iDataName) 1 

# make sure file can be opened
	if { [catch {open $fn r} fileId] } {
		u_error "Cannot open input file '$fn'."
	}

# read each line
	while { [gets $fileId rline] >= 0 } {
		set line [string trim $rline]
		if {[string index $line 0] != $xcv(data,comment)} {
			set nd [scan $line "%s %s %s %s %s %s %s %s " s1 s2 s3 s4 s5 s6 s7 errMsg]

			# process max of 7 fields 
			if {[string length $errMsg] > 0} {
				u_error "Illegal data format at line: \n$line."
			}

			# skip blank lines
			if {$nd < 0} {
				continue
			}

			# format error
			if {$nd == 0} {
				u_error "Data format (bad 1st arg) at: \n '$line'."
			}

			# one argument only, assume new dataset
			if {$nd == 1} {
				scan $line "%s" name 

				# tidy up last good dataset read
				if {$iDataNum > 0} {
					set newData($iSetNum,iDataNum) $iDataNum
					incr iSetNum
					set iDataNum 0
				} else {
					if {$iSetNum > 0} {
						u_error "No data points at: \n '$line'."
					}
				}
				set newData($iSetNum,iDataName) $name 
				continue
			}

			# else process data lines
			set nd [scan $line "%f %f %f %f %f %f %f" xd yd td xlb xub ylb yub]

			# start with basic x,y data and no data measurement errors. 
			set newData($iSetNum,$iDataNum,xd) $xd
			set newData($iSetNum,$iDataNum,yd) $yd
			set newData($iSetNum,$iDataNum,td) 0.0 
			set newData($iSetNum,$iDataNum,xlb) 0.0 
			set newData($iSetNum,$iDataNum,xub) 0.0
			set newData($iSetNum,$iDataNum,ylb) 0.0
			set newData($iSetNum,$iDataNum,yub) 0.0
			lappend xItems($iSetNum) $xd

			# third data field (td) found with no measurement errors entered 
			# td cannot be zero
			if {$nd == 3} {
				if {$td == 0.0} {
					u_error "Illegal data format (nd=3,td=0) at: \n$line."
				}
				set newData($iSetNum,$iDataNum,td) $td
				set xd [expr $xd / $td]
			}

			# no third data field (td) found, but measurement errors entered 
			# remember columns are shifted by one.
			if {$nd == 6} {
				set nd 2
				set newData($iSetNum,$iDataNum,xlb) [expr $xd - $td]
				set newData($iSetNum,$iDataNum,xub) [expr $xd + $xlb]
				set newData($iSetNum,$iDataNum,ylb) [expr $yd - $xub]
				set newData($iSetNum,$iDataNum,yub) [expr $yd + $ylb]
			}

			# third data field (td) found, and measurement errors entered 
			if {$nd == 7} {
				if {$td == 0.0} {
					u_error "Illegal data format (nd=5,td=0) at: \n$line."
				}
				set newData($iSetNum,$iDataNum,td) $td
				set xd [expr $xd / $td]
				set nd 3
				set newData($iSetNum,$iDataNum,xlb) [expr $xd - $xlb]
				set newData($iSetNum,$iDataNum,xub) [expr $xd + $xub]
				set newData($iSetNum,$iDataNum,ylb) [expr $xd - $ylb]
				set newData($iSetNum,$iDataNum,yub) [expr $xd + $yub]
			}
		
			if {$xMin == ""} {
				set xMin $xd 
				set xMax $xd
				set yMin $yd 
				set yMax $yd
			} else {
				set xMax [get_max $xMax $xd]
				set yMax [get_max $yMax $yd]
				set xMin [get_min $xMin $xd]
				set yMin [get_min $yMin $yd]
			}
			set newData($iSetNum,nCol) $nd
			incr iDataNum 
			continue
		}
	}
	close $fileId

# tidy up last dataset read
	if {$iDataNum > 0} {
		set newData($iSetNum,iDataNum) $iDataNum
		incr iSetNum
	} else {
		u_error "Check data input format, no data points at: \n '$line'."
	}

# did we read in the data ok
	for {set iSet 0} {$iSet < $iSetNum} {incr iSet} {
		if { $newData($iSet,iDataNum) <= 1 } {
			u_error "Dataset $iSet has fewer than 2 data points"
		}
	}

	set myData($iFile,bounded_left) $xcv(X_MAX)
	set myData($iFile,bounded_right) $xcv(X_MIN)
	for {set iSet 0} {$iSet < $iSetNum} {incr iSet} {
		set myData($iFile,$iSet,iDataNum) $newData($iSet,iDataNum)
		set myData($iFile,$iSet,iDataName) $newData($iSet,iDataName)
		set myData($iFile,$iSet,nCol) $newData($iSet,nCol)

		# store data in ascending order 
		set n $myData($iFile,$iSet,iDataNum)
		set xsort [lsort -real $xItems($iSet)]
		for {set iData 0} {$iData < $n} {incr iData} {
			set ix [lsearch $xItems($iSet) [lindex $xsort $iData]] 
			set myData($iFile,$iSet,$iData,xd) $newData($iSet,$ix,xd)
			set myData($iFile,$iSet,$iData,yd) $newData($iSet,$ix,yd)
			set myData($iFile,$iSet,$iData,td) $newData($iSet,$ix,td)
			set myData($iFile,$iSet,$iData,ylb) $newData($iSet,$ix,ylb)
			set myData($iFile,$iSet,$iData,yub) $newData($iSet,$ix,yub)
			set myData($iFile,$iSet,$iData,xlb) $newData($iSet,$ix,xlb)
			set myData($iFile,$iSet,$iData,xub) $newData($iSet,$ix,xub)
		}

		set myData($iFile,$iSet,gridFlag) "FALSE"
		set aMin [bounded_convert $iFile $iSet 0]
		set myData($iFile,$iSet,bounded_left) $aMin
		if { $aMin < $myData($iFile,bounded_left) } {
			set myData($iFile,bounded_left) $aMin
		}
		set aMax [bounded_convert $iFile $iSet [expr $n - 1]]
		set myData($iFile,$iSet,bounded_right) $aMax
		if { $aMax > $myData($iFile,bounded_right) } {
			set myData($iFile,bounded_right) $aMax
		}
	}

	# Start at zero for xMin/yMin if we are close to it
	if {$xMin > 0} {
		set ratio [expr ($xMax - $xMin) / $xMin]
		if { $ratio >= 2.0 } {
			set xMin 0.0
		}
	}
	if {$yMin > 0} {
		set ratio [expr ($yMax - $yMin) / $yMin]
		if { $ratio >= 2.0 } {
			set yMin 0.0
		}
	}

	u_log "Read data '$fn', Number of datasets: '$iSetNum'."

	set myData($iFile,fileName) $fn
	set myData($iFile,iSetNum) $iSetNum
	set myData($iFile,x_min) $xMin
	set myData($iFile,x_max) $xMax
	set myData($iFile,y_min) $yMin
	set myData($iFile,y_max) $yMax
}

#####################################################################
proc find_file_type { fn } {
	global panvar xcv

# cannot open data file
	if [catch {open $fn r} fileId] {
		set panvar(data,format) unreadable
		return
	}

	if [info exists nd1] { unset nd1 }
	if [info exists nd2] { unset nd2 }

# get first two data lines 
	while {[gets $fileId rline] >= 0} {
		set line [string trim $rline]
		if {$line == ""} {
			continue
		}
		if {[string index $line 0] == $xcv(data,comment)} {
			continue
		}
		if ![info exists nd1] {
			set nd1 [scan $line "%s %s %s %s %s %s %s" x1 x2 x3 x4 x5 x6 x7]
			if {$x1 == "List"} {
				set panvar(data,format) fp
				close $fileId
				return
			}
			continue
		}
		set nd2 [scan $line "%s %s %s %s %s %s %s %s" y1 y2 y3 y4 y5 y6 y7 y8]
		break
	}
	close $fileId

	if {$nd1 == 1} {
		if {$nd2 == 1} {
			set panvar(data,format) jfit
			return
		}
	}

	# kay format must have minimum 3 points for fitting
	# second line will have one more data field than first line
	if { $nd2 > 7 } {
		set panvar(data,format) kay 
		return
	}
	incr nd1
	if { $nd1 == $nd2 && $nd1 > 3 } {
		set panvar(data,format) kay 
		return
	}
	set panvar(data,format) crvfit
}
