
#####################################################################
#
# xcrvfit help screen
#
#####################################################################

#####################################################################
#
# event procedure for viewing function help files 
#
proc bind_func_desc { } {
	global panvar xcv myFunc

	set xcv(help,README) readme 
	set xcv(help,DATA) data 
	set xcv(help,OUTPUT) output
	set xcv(help,PLOT) plot
	set xcv(help,FILE_TYPES) "$xcv(help,README) $xcv(help,DATA) $xcv(help,OUTPUT) $xcv(help,PLOT)"
	set panvar(help,fileType) [lindex $xcv(help,FILE_TYPES) 0] 

	set f ".$xcv(win,help)"
	if [winfo exists $f] { return }
	toplevel $f

	set f $f.fr1
	frame $f
	pack $f -pady 10
	set m $f.select
	frame $m
	pack $m 
	label $m.label -text "Fit Function:" -width 12
	button $m.next -text Next \
		-command "help_inc $f $f.select.func vw 1"
	button $m.prev -text Prev \
		-command "help_inc $f $f.select.func vw -1"
	menubutton $m.func -padx 9 -menu $m.func.menu -underline 0 -width 20
	pack $m.label $m.func $m.next $m.prev -side left
	menu $m.func.menu
	set count 0
	foreach group $xcv(function,list) {
		$m.func.menu add cascade -label $group -menu $m.func.menu.$group
		menu $m.func.menu.$group
		foreach type $xcv(function,$group) {
			$m.func.menu.$group add command -label $myFunc($type,name) \
				-command "help_func_select $f $f.select.func vw $type $count"
			incr count
			lappend panvar(help,functions) $type
		}
	}
	set panvar(help,functions,num) [expr $count - 1 ]

	frame $f.ftype 
	pack $f.ftype -pady 5 
	label $f.ftype.label -text "Help File:" -width 12
	pack $f.ftype.label -side left
	set count 0
	foreach i $xcv(help,FILE_TYPES) {
		radiobutton $f.ftype.$count -text $i -anchor w \
			-variable panvar(help,fileType) -value $i \
			-command "help_func_load $f vw"
		pack $f.ftype.$count -anchor w -side left -padx 5
		incr count
	}

	set_text_view $f.vw
	help_func_select $f $f.select.func vw [lindex $panvar(help,functions) 0] 0
}


proc set_text_view { f } {
	set ht 30
	if ![winfo exists $f] {
		frame $f
		pack $f
	}
   	text $f.text -relief raised -bd 2 -state normal -height $ht \
		-yscrollcommand "$f.scroll set"
	scrollbar $f.scroll -command "$f.text yview"
	pack $f.scroll -side right -fill y -pady 10
	pack $f.text -side left -pady 10
}

proc help_func_select { f mSelect tview key count } {
	global myFunc panvar

	$mSelect configure -text $myFunc($key,name)
	set panvar(help,key) $key
	set panvar(help,key,index) $count
	help_func_load $f $tview 
}

proc help_inc { f mSelect tview inc } {
	global panvar

	set count [ expr $panvar(help,key,index) + $inc ]
	if { $count < 0 } {
		set count $panvar(help,functions,num)
	}
	if { $count > $panvar(help,functions,num) } {
		set count 0
	}
	set newkey [lindex $panvar(help,functions) $count] 
	help_func_select $f $mSelect $tview $newkey $count
}

proc help_func_load { f tview } {
	global panvar xcv
	package require Img

	set loadFrame $f.$tview
	set key $panvar(help,key)

	set searchdir $xcv(fn,desc)/$key
	if { $panvar(help,fileType) == $xcv(help,DATA) } {
		set files [ glob -nocomplain $searchdir/test*.data ]
	}
	if { $panvar(help,fileType) == $xcv(help,README) } {
		set files [ glob -nocomplain $searchdir/readme*.txt ]
	}
	if { $panvar(help,fileType) == $xcv(help,OUTPUT) } {
		set files [ glob -nocomplain $searchdir/out*.txt ]
	}
	if { $panvar(help,fileType) == $xcv(help,PLOT) } {
		set png [ glob -nocomplain $searchdir/out*.png ]
		set jpeg [ glob -nocomplain $searchdir/out*.jpeg ]
		set tiff [ glob -nocomplain $searchdir/out*.tiff ]
		set gif [ glob -nocomplain $searchdir/out*.gif ]
		set files "$png $jpeg $tiff $gif" 
	}
	set fn [lindex $files 0]

	# help file does not exist
	if {$fn == "" || ![file readable $fn]} {
		if ![winfo exists $loadFrame.text] {
			destroy $loadFrame.label
			set_text_view $loadFrame
		}
		$loadFrame.text delete 1.0 end
		$loadFrame.text insert end "No $panvar(help,fileType) file for '$key'." 
		return
	}

	# plots are handled differently 
	if { $panvar(help,fileType) == $xcv(help,PLOT) } {
		if [winfo exists $loadFrame.text] {
			destroy $loadFrame.text
			destroy $loadFrame.scroll
			label $loadFrame.label
			pack $loadFrame.label -pady 10
		} 
		image create photo myImage -file $fn
		$loadFrame.label configure -image myImage
		return
	}

	# regular text output file 
	if [winfo exists $loadFrame.label] {
		destroy $loadFrame.label
		set_text_view $loadFrame
	}
	$loadFrame.text delete 1.0 end
	set f [open $fn]
	while {![eof $f]} {
		$loadFrame.text insert end [read $f 1000]
	}
	close $f
}
