
#####################################################################
#
# xcrvfit mfit window 
#
#####################################################################

#####################################################################
#
# Create main panel for mfit
#
proc do_mfit { } {
	global xcv myData vPlot panvar

	set pid 4 
	set iFile 0
	set mix 0

	for {set np 0} {$np < $panvar(parms,num)} {incr np} {
		set panvar(mfit,choice,$np) $panvar(choice,$np) 
	}

	# show mfit-select plot with first dataset
	plot_newdata $pid $mix $iFile
	set myData($iFile,$mix,gridFlag) "TRUE"
	make_mfit_panel $pid
}

#####################################################################
#
# Create main panel for mfit
#
proc make_mfit_panel { pid } {
	global xcv vPlot panvar myData myFunc

	set panvar(mfit,colWidth) [expr 3 + [string length $myData(0,0,iDataName)]]
	set panvar(mfit,runCount) 0

	set f ".$xcv(win,mfit,main)"
	set tab $f.table
	set databar $f.databar
	set parm $f.parm

	toplevel $f

	frame $databar
	pack $databar -pady 10
	label $databar.label -text "DataSets: "
	button $databar.all -text "Select All" \
		-command "mfit_select_all $pid $tab TRUE"
	button $databar.none -text "Select None" \
		-command "mfit_select_all $pid $tab FALSE"
	pack $databar.label $databar.all $databar.none -anchor w -side left

	frame $tab
	pack $tab
	set rowNum 0
	set colNum [expr $vPlot($pid,col_max) + 1] 
	for {set ix 0 } {$ix < $myData(0,iSetNum)} {incr ix} {
		if {$colNum > $vPlot($pid,col_max)} {
			incr rowNum
			frame $tab.$rowNum -padx 20
			pack $tab.$rowNum -anchor w
			set colNum 1
		}
		set panvar(mfit,$ix,row) $rowNum
		set panvar(mfit,$ix,col) $colNum
		set resBut $tab.$rowNum.$colNum
		label $resBut -text $myData(0,$ix,iDataName) \
			-relief raised -pady 4 -width $panvar(mfit,colWidth) 
		$tab.$rowNum.$colNum configure -background $vPlot($pid,no_color) 
		bind $resBut <Button-1> "mfit_select_data $pid $tab.$rowNum.$colNum $ix"
		if { $myData(0,$ix,gridFlag) == "FALSE" } {
			$resBut configure -background $vPlot($pid,no_color) 
		} else {
			$resBut configure -background $vPlot($pid,yes_color) 
		}
		pack $resBut -side left
		incr colNum
	}

	frame $parm
	pack $parm -pady 10
	frame $parm.title
	pack $parm.title -padx 10
	label $parm.title.1 -text "Parameter" -width 14
	label $parm.title.2 -text "Starting_Value" -width 12
	label $parm.title.3 -text "Hold_Constant" -width 18
	pack $parm.title.1 $parm.title.2 $parm.title.3 \
		-ipadx 0 -padx 0 -side left

	for {set np 0} {$np < $panvar(parms,num)} {incr np} {

		frame $parm.p$np
		pack $parm.p$np -padx 10

		set pLab [lindex $myFunc($panvar(functionKey),parms) $np]
		label $parm.p$np.item -text $pLab -width 8

		command_entry $parm.p$np.start "" 0 "bind_parm $np" \
			-textvar panvar(newParms,$np) -width 12 
		$parm.p$np.start.entry configure -background $vPlot($pid,no_color) 

		set cf [frame $parm.p$np.choice -borderwidth 0]
		radiobutton $cf.0 -variable panvar(mfit,choice,$np) \
			-text "yes" -value "yes"
		pack $cf.0 -side left 
		radiobutton $cf.1 -variable panvar(mfit,choice,$np) \
			-text "no" -value "no"
		pack $cf.1 -side left

		pack $parm.p$np.item $parm.p$np.start -side left
		pack $cf -padx 5 -side left
	}

	frame $f.but
	pack $f.but -pady 10 -padx 20
	button $f.but.start -text "Go" -command "mfit_calc"
	button $f.but.quit -text "Cancel" -command "destroy $f"
	pack $f.but.start $f.but.quit -side left 
}

#####################################################################
#
# User clicks on one of the datasets 
#
proc mfit_select_data { pid item iSet } {
	global myData panvar vPlot xcv

	set pid 4

	# update panel
	if { $myData(0,$iSet,gridFlag) == "FALSE" } {
		set myData(0,$iSet,gridFlag) "TRUE" 
		$item configure -background $vPlot($pid,yes_color) 
	} else {
		set myData(0,$iSet,gridFlag) "FALSE" 
		$item configure -background $vPlot($pid,no_color) 
	}

	# update our plot parameters 
	set current_data -1
	set count 0
	set pNum 0
	for {set mix 0 } {$mix < $myData(0,iSetNum)} {incr mix} {
		if { $myData(0,$mix,gridFlag) == "TRUE" } {
			plot_set_data $pid $pid $pNum $count 0 $mix
			# hilite a dataset when it is added to the list 
			if { $mix == $iSet } {
				set vPlot($pid,0,$count,data_color) $vPlot($pid,data_color)
				set current_data $count 
			} else {
				set vPlot($pid,0,$count,data_color) $vPlot($pid,data_color_opt)
				set vPlot($pid,0,$count,data_label_text) "" 
			}
			incr count
		}
	}
	set vPlot($pid,0,dNumTot) $count
	update

	# draw what we have
	do_plot $pid 0

	# draw current data last and in a different color so that it is easy to see 
	if { $current_data >= 0 } { 
		set vPlot($pid,0,$current_data,data_color) $vPlot($pid,data_color)
		draw_data $pid 0 $current_data 0
	}
}

#####################################################################
#
# Select all or none of the datasets for grid fitting
# 
proc mfit_select_all { pid tab val } {
	global myData panvar vPlot xcv

	set pid 4

	# update panel
	for {set ix 0 } {$ix < $myData(0,iSetNum)} {incr ix} {
		set myData(0,$ix,gridFlag) $val
		set rowNum $panvar(mfit,$ix,row)
		set colNum $panvar(mfit,$ix,col)
		if {$val == "TRUE"} {
			$tab.$rowNum.$colNum configure -background $vPlot($pid,yes_color) 
		} else {
			$tab.$rowNum.$colNum configure -background $vPlot($pid,no_color) 
		}
	}

	# update our plot parameters 
	set count 0
	set pNum 0
	for {set mix 0 } {$mix < $myData(0,iSetNum)} {incr mix} {
		if { $myData(0,$mix,gridFlag) == "TRUE" } {
			plot_set_data $pid $pid $pNum $count 0 $mix
			set vPlot($pid,0,$count,data_color) $vPlot($pid,data_color_opt)
			set vPlot($pid,0,$count,data_label_text) "" 
			incr count
		}
	}
	set vPlot($pid,0,dNumTot) $count
	update

	# draw what we have
	do_plot $pid 0
}

#####################################################################
#
# User hits the Go button
#
proc mfit_calc { } {
	global myFunc panvar myData vPlot xcv

	set f ".$xcv(win,mfit,calc)"

	u_log " -> Gridfit Button"
	incr panvar(mfit,runCount)

	# Get copy of parameter state
	set panvar(parmGrid) "" 
	for {set np 0} {$np < $panvar(parms,num)} {incr np} {
		if [info exists panvar(parms,$np)] {
			set panvar(parmsCopy,$np) $panvar(parms,$np) 
		}
	}

	set dataFlag 0
	for {set ix 0 } {$ix < $myData(0,iSetNum)} {incr ix} {
		if { $myData(0,$ix,gridFlag) == "TRUE" } {
			set dataFlag 1
			break
		}
	}
	if { $dataFlag == 0 } {
		u_error "No dataSets selected"
	}

	# Show output panel if we have selected data
	show_mfit_results_panel $f

	# data output labels
	set constLine ""
	set parmLine ""
	for {set np 0} {$np < $panvar(parms,num)} {incr np} {
		set panvar(mfit,$np,dCount) 0
		if {[string compare $panvar(mfit,choice,$np) no] == 0} {
	    	set lab [lindex $myFunc($panvar(functionKey),parms) $np]
			set lab [format "%8s %7s " $lab " "]
			set parmLine "$parmLine $lab"
		} else {
	    	set lab [lindex $myFunc($panvar(functionKey),parms) $np]
			set val "$panvar(newParms,$np)"
			set constLine "$constLine $lab=$val  "
		}
	}
	if { $constLine == "" } {
		set constLine " none"
	}
	set tf $f.results
	$tf.text insert end "\n"
	$tf.text insert end "   Fitting Results #$panvar(mfit,runCount)\n"
	$tf.text insert end "\n"
	$tf.text insert end "Fitting Function: $panvar(functionKey)\n"
	$tf.text insert end "Parameter Constants: $constLine\n"
	$tf.text insert end "\n"
	$tf.text insert end "Dataset    RMSE   $parmLine\n"

	# process each dataset
	set pid 5
	set pNum 0
	set gridParmList [mfit_set_parms]
	for {set ix 0 } {$ix < $myData(0,iSetNum)} {incr ix} {
		if { $myData(0,$ix,gridFlag) == "TRUE" } {

			set vPlot($pid,$pNum,okFlag) "TRUE"
			set vPlot($pid,$pNum,dNumCur) 0
			set vPlot($pid,$pNum,dNumTot) 1

			set vPlot($pid,$pNum,0,mix) $ix
			u_log "DataSet: $myData(0,$ix,iDataName)"
			set panvar(dataLabel) $myData(0,$ix,iDataName)
			write_crvfit_data $pid $pNum 0 $xcv(fn,dataTmp)
			set panvar(mfit,dataIx,$pNum) $ix

			set rc [mfit_evaluate $pNum $gridParmList]

			# unable to achieve convergence. How about manually 
			if { $rc == "FALSE" } {
				set rc [mfit_manual $pNum $gridParmList]
			}

			# show results for this dataset
			mfit_update_panel $f $rc $panvar(dataLabel) $pNum $gridParmList

			incr pNum 
		}
	}
	set vPlot($pid,pNumTot) $pNum

	# summarize and save results to a file
	mfit_analyze $f $pNum
	save_text $f.results.text $xcv(fn,resGridTmp)
}

#####################################################################
#
# Display best estimte of the variable function parameters 
#
proc mfit_propose { np mean } {
	global myFunc panvar vPlot xcv

	set tStat 0.0
	for { set ix 0 } {$ix < $panvar(mfit,$np,dCount)} {incr ix} {
		set x $panvar(mfit,parmVal,$np,$ix)
		set err [expr abs(($x - $mean) / $panvar(mfit,stddev,$np,$ix)) ]
		set tStat [expr $tStat + $err]
	}
	return $tStat
}

#####################################################################
#
# Display best estimte of the variable function parameters 
#
proc mfit_analyze { f dNumTot } {
	global myFunc panvar vPlot xcv

	# sort and find median parameter values
	for {set np 0} {$np < $panvar(parms,num)} {incr np} {
		if { $panvar(mfit,$np,dCount) == 0 } {
			continue
		}
		for {set ix 0 } {$ix < $panvar(mfit,$np,dCount)} {incr ix} {
			lappend parmVal($np) $panvar(mfit,parmVal,$np,$ix)
		}
		set parmSorted($np) [lsort -real $parmVal($np)]
		set mid [expr $panvar(mfit,$np,dCount) / 2]
		set parmVal(mid,$np) [lindex $parmSorted($np) $mid]
		set parmVal(lo,$np) [expr $parmVal(mid,$np) * 0.05]
		set parmVal(hi,$np) [expr $parmVal(mid,$np) * 2]

		set stddev(lo,$np) [mfit_propose $np $parmVal(lo,$np)]
		set stddev(hi,$np) [mfit_propose $np $parmVal(hi,$np)]

		set convTerm [expr $parmVal(mid,$np) * 0.001]
		while { "TRUE" } {
			set interval [expr $parmVal(hi,$np) - $parmVal(lo,$np)]
			if {$interval < $convTerm} {
				break
			}

			set mlo [expr $parmVal(lo,$np) + (0.333 * $interval) ]
			set slo [mfit_propose $np $mlo]

			set mhi [expr $parmVal(lo,$np) + (0.667 * $interval) ]
			set shi [mfit_propose $np $mhi]

			if { $slo < $shi } {
				set parmVal(hi,$np) $mhi
				set stddev(hi,$np) $shi
			} else {
				set parmVal(lo,$np) $mlo
				set stddev(lo,$np) $slo
			}
		}
	    set lab [lindex $myFunc($panvar(functionKey),parms) $np]
		set outLine "Estimated value of $lab: $parmVal(hi,$np)"
		$f.results.text insert end "$outLine\n"
	}
	return TRUE
}

proc mfit_manual { dNum gridParmList } {
	global panvar xcv myFunc myData

	# could we set boundaries on selected parameter
	set rc [mfit_set_limits $dNum $gridParmList]
	if { $rc == "FALSE" } {
		return FALSE
	}

	# Now we use bisection until we achieve convergence. 
	set convTerm [expr $panvar(mid) * 0.01]
	set np $panvar(mfitParm)
	set convergeFlag "TRUE"
	while { "TRUE" } {

		set interval [expr $panvar(hi) - $panvar(lo)]
		if {$interval < $convTerm} {
			break
		}

		set panvar(mlo) [expr $panvar(lo) + 0.333 * $interval]
		set gridParmList [lreplace $gridParmList $np $np =$panvar(mlo)]
		set rc [mfit_evaluate $dNum $gridParmList]
		set panvar(rmse,mlo) $panvar(mfit,outrmse,$dNum)

		set panvar(mhi) [expr $panvar(lo) + 0.667 * $interval]
		set gridParmList [lreplace $gridParmList $np $np =$panvar(mhi)]
		set rc [mfit_evaluate $dNum $gridParmList]
		set panvar(rmse,mhi) $panvar(mfit,outrmse,$dNum)

		if { $panvar(rmse,mlo) < $panvar(rmse,mhi) } {
			set panvar(hi) $panvar(mhi)
			set panvar(rmse,hi) $panvar(rmse,mhi)
		} else {
			set panvar(lo) $panvar(mlo)
			set panvar(rmse,lo) $panvar(rmse,mlo)
		}
	}
	return TRUE 
}

proc mfit_set_limits { dNum gridParmList } {
	global panvar xcv myFunc myData

	set maxCount 7
	set panvar(rmse,lo) 0
	set panvar(rmse,hi) 0

	set np $panvar(mfitParm)
	set panvar(mid) [lindex $gridParmList $np]
	set panvar(lo) $panvar(mid)
	set panvar(hi) $panvar(mid)

	set gridParmList [lreplace $gridParmList $np $np =$panvar(mid)]
	set rc [mfit_evaluate $dNum $gridParmList]
	set panvar(rmse,mid) $panvar(mfit,outrmse,$dNum)

	set count 0
	while { $panvar(rmse,lo) < $panvar(rmse,mid) } {
		set panvar(lo) [ expr 0.5 * $panvar(lo) ]
		set gridParmList [lreplace $gridParmList $np $np =$panvar(lo)]
		set rc [mfit_evaluate $dNum $gridParmList]
		set panvar(rmse,lo) $panvar(mfit,outrmse,$dNum)
		if { $count >= $maxCount } {
	    	set lab [lindex $myFunc($panvar(functionKey),parms) $np]
			set panvar(errStr) \
				"Cannot find a starting lower bound for parameter: $lab<$panvar(lo)" 
			return FALSE
		}
		incr count
	}

	set count 0
	while { $panvar(rmse,hi) < $panvar(rmse,mid) } {
		set panvar(hi) [ expr 2.0 * $panvar(hi) ]
		set gridParmList [lreplace $gridParmList $np $np =$panvar(hi)]
		set rc [mfit_evaluate $dNum $gridParmList]
		set panvar(rmse,hi) $panvar(mfit,outrmse,$dNum)
		if { $count >= $maxCount } {
	    	set lab [lindex $myFunc($panvar(functionKey),parms) $np]
			set panvar(errStr) \
				"Cannot find a starting upper bound for parameter: $lab>$panvar(hi)" 
			return FALSE
		}
		incr count
	}
	return TRUE
}

proc mfit_set_parms { } {
	global panvar xcv myFunc myData

	set gridParmList ""
	set panvar(mfitParm) -1
	for {set np 0} {$np < $panvar(parms,num)} {incr np} {

		if ![info exists panvar(newParms,$np) ] {
			u_error "No value given for parameter $np"
		}

		set val [string trim $panvar(newParms,$np)]

		if {[string length $val] == 0} {
			u_error "No value given for parameter $np"
		}

		# Current parameter is held constant, append "=".
		if {[string compare $panvar(mfit,choice,$np) yes] == 0} {
			set val "=$val"
		} else {
			if { $panvar(mfitParm) < 0 } {
				set panvar(mfitParm) $np
			}
		}
		lappend gridParmList $val 
	}
	return $gridParmList
}

#####################################################################
#
# we have a set of parameters in order to call fastcrvfit 
# keep track of the results
#
proc mfit_evaluate { dNum gridParmList } {
	global panvar xcv myFunc myData

	# we have a complete parameter list, call fastcrvfit
	if { [call_fastcrvfit $gridParmList] == "FALSE" } {
		u_log "Unable to converge for DataSet: $myData(0,$dNum,iDataName)"
		return FALSE
	}

	# read best parameters from fastcrvfit and store them
	set f [open $xcv(fn,resShortTmp)]
	set outLine "Output: "
	for {set np 0} {$np < $panvar(parms,num)} {incr np} {
		gets $f line
		set nd [scan $line "%f %f" panvar(mfit,outparms,$dNum,$np) panvar(mfit,outstddev,$dNum,$np)]
		set outLine "$outLine $panvar(mfit,outparms,$dNum,$np) $panvar(mfit,outstddev,$dNum,$np)"
	}

	gets $f line
	set nd [scan $line "%f" panvar(mfit,outrmse,$dNum)]
	close $f
	set outLine "$outLine rmse=$panvar(mfit,outrmse,$dNum)"
	u_log $outLine
	return TRUE
}

#####################################################################
#
# show how each data fits with the best parms from mfit 
#
proc mfit_show_fits { } {
	global panvar vPlot myData xcv

	set iFile 0 
	set pid 5
	set pNum 0
	set vPlot($pid,pNumTot) 0
	set vPlot($pid,pNumCur) 0
	create_plot_win $pid

	# process each dataset
	for {set mix 0 } {$mix < $myData(0,iSetNum)} {incr mix} {
		if { $myData(0,$mix,gridFlag) == "TRUE" } {

			set vPlot($pid,$pNum,okFlag) "TRUE"
			plot_init_new $pid $pid $pNum $iFile 
			plot_set_data $pid $pid $pNum 0 0 $mix
			incr vPlot($pid,pNumTot)
			set vPlot($pid,$pNum,dNumCur) 0
			set vPlot($pid,$pNum,dNumTot) 1

			set vPlot($pid,$pNum,0,mix) $mix
			u_log "DataSet: $myData(0,$mix,iDataName)"
			set panvar(dataLabel) $myData(0,$mix,iDataName)
			set panvar(mfit,dataIx,$pNum) $mix
			for {set np 0} {$np < $panvar(parms,num)} {incr np} {
				set panvar(newParms,$np) $panvar(mfit,outparms,$pNum,$np)
				set panvar(parms,$np) $panvar(newParms,$np)
			}
			calc_fit_one $pid $pNum 0
			incr pNum 
		}
	}

	find_grid $pid 0
	plot_set_grid $pid $vPlot($pid,grid,row) $vPlot($pid,grid,col)
	calibrate_xy $pid
	do_plot $pid $iFile

	raise ".$xcv(win,plot,$pid)"

	# Put original parameters back
	for {set np 0} {$np < $panvar(parms,num)} {incr np} {
		if [info exists panvar(parmsCopy,$np)] {
			set panvar(parms,$np) $panvar(parmsCopy,$np) 
			set panvar(newParms,$np) $panvar(parmsCopy,$np) 
		}
	}
}


#####################################################################
#
# Gridfit results panel 
#
proc show_mfit_results_panel { f } {
	global xcv panvar myFunc

	if [winfo exists $f] {
		raise $f
		return
	}

	set tf $f.results
	toplevel $f

	frame $tf 
	pack $tf -pady 10 -padx 20 -expand true -fill both
	text $tf.text -relief raised -width 90 \
		-height 24 -bd 2 -yscrollcommand "$tf.scroll set"
	scrollbar $tf.scroll -command "$tf.text yview"
	pack $tf.scroll -side right -fill y
	pack $tf.text -side left -expand true -fill both

	frame $f.but
	pack $f.but -pady 10 -padx 20
	set outFile $xcv(fn,resGridTmp)
	button $f.but.save -text "Save to File" \
		-command "bind_save_file $outFile"
	button $f.but.show -text "Show Fit for each Dataset" \
		-command "mfit_show_fits"
	button $f.but.quit -text "Dismiss" -command "destroy $f"
	pack $f.but.save $f.but.show $f.but.quit -side left
}

#####################################################################
#
# Show progress window to display calculation results
#
proc mfit_update_panel { f rc dataset dNum gridParmList } {
	global panvar myFunc

	set tf $f.results

	# create summary line for this dataset 
	if { $rc == "FALSE" } {
		set outLine [format "%-6s %s" \
			$panvar(dataLabel) $panvar(errStr)]
		$tf.text insert end "$outLine\n"
		return
	}

	set hd [format "%-6s %8.5g" \
		$panvar(dataLabel) $panvar(mfit,outrmse,$dNum)]
	set outLine "$hd  "

	# add parameter values
	for {set np 0} {$np < $panvar(parms,num)} {incr np} {
		if {[string compare $panvar(mfit,choice,$np) no] == 0} {
			set val [format "%8.5g" $panvar(mfit,outparms,$dNum,$np)]
			set stddev [format "%-8.5g" $panvar(mfit,outstddev,$dNum,$np)]
			if {$stddev == 0} {
				set outLine "$outLine $val --------"
			} else {
				set outLine "$outLine $val $stddev"
				set panvar(mfit,parmVal,$np,$panvar(mfit,$np,dCount)) $val
				set panvar(mfit,stddev,$np,$panvar(mfit,$np,dCount)) $stddev
				incr panvar(mfit,$np,dCount)
			}
		}
	}
	# show output line
	$tf.text insert end "$outLine\n"
	update
}
