
#####################################################################
#####################################################################
#
# Plot settings
#
#####################################################################

#####################################################################
# 
# Graphics options window
#
proc plot_settings { pid f } {
	global panvar vPlot xcv

	if ![winfo exists .$xcv(win,plot,$pid)] {
		u_error "Window '$xcv(win,plot,$pid)' does not exist"
	}

	set panvar(plotWinId) $pid
	set padx 10
	if ![winfo exists $f] {
		toplevel $f
	} else {
		raise $f
	}

	# if overlay window exists, set plot mode to DATA.
	if [winfo exists .$xcv(win,overlay)] {
		set panvar($pid,settings) DATA 
	} else {
		set panvar($pid,settings) ALL
	}

	set panvar(x_max) [format "%-8.5g" $panvar(x_max)]
	set panvar(y_max) [format "%-8.5g" $panvar(y_max)]
	set panvar(x_min) [format "%-8.5g" $panvar(x_min)]
	set panvar(y_min) [format "%-8.5g" $panvar(y_min)]

	set nfr $f.0
	frame $nfr
	pack $nfr -pady 20
	label $nfr.label -text "Apply changes to: "
	radiobutton $nfr.data -variable panvar($pid,settings) \
		-text "Current Data   " -value DATA 
	radiobutton $nfr.plot -variable panvar($pid,settings) \
		-text "Current Plot   " -value PLOT 
	radiobutton $nfr.all -variable panvar($pid,settings) \
		-text "All Plots" -value ALL
	pack $nfr.label -padx 30 -side left
	pack $nfr.data $nfr.plot $nfr.all -side left

	set nfr $f.1
	frame $nfr
	pack $nfr -anchor nw
	set_background $nfr.1 $pid "Background Color" panvar(canvas_bg_color) \
		$vPlot($pid,canvas) canvas_bg_color 25
	set_color $nfr.2 $pid "Axis Color" 25 vPlot($pid,axis_color) "axis_color"
	pack $nfr.1 $nfr.2 -padx $padx -anchor nw -side left

	set nfr $f.2
	frame $nfr
	pack $nfr -anchor nw
	set_color $nfr.1 $pid "Data Color" 25 panvar(data_color) "data_color"
	set_color $nfr.2 $pid "Fitting Function" 25  panvar(fit_color) "fit_color"
	pack $nfr.1 $nfr.2 -padx $padx -anchor nw -side left

	set nfr $f.3
	frame $nfr
	pack $nfr -anchor nw
	set_color $nfr.1 $pid "Data Label Color" 25 \
		panvar(data_label_color) "data_label_color"
	set_radio $nfr.2 $pid "Data Symbol" 25 "data_symbol" "data_symbol"
	pack $nfr.1 $nfr.2 -padx $padx -anchor nw -side left

	set nfr $f.4
	frame $nfr
	pack $nfr -anchor nw
	set_radio $nfr.1 $pid "Show Data Error Bars" 25 "yes_no" "data_error_bars"
	pack $nfr.1 -padx $padx -anchor nw -side left

	label $f.nop0 -text " "
	pack $f.nop0 -side top

	set nfr $f.5
	frame $nfr
	pack $nfr 
	set_font_attr $pid $nfr.1 "Font Name" 15 panvar(font_name) "font_name" 
	set_font_attr $pid $nfr.2 "Font Style" 15 panvar(font_style) "font_style" 
	set_font_attr $pid $nfr.3 "Font Size" 15 panvar(font_size) "font_size" 
	pack $nfr.1 $nfr.2 $nfr.3 -padx $padx -side left

	label $f.nop1 -text " "
	pack $f.nop1 -side top

	set tWidth 15
	set textLen 20
	set oWidth 4
	set xPad 10
	set lPad 50

	set nfr $f.labels1
	frame $nfr
	pack $nfr -anchor nw -padx $lPad
	command_entry $nfr.gleft  "Graph Label: " 15 \
		"bind_textbox $pid graph_label_text 0" \
		-textvar panvar(graph_label_text) -width $textLen 
	command_entry $nfr.gright  "Data Label: " 15 \
		"bind_textbox $pid data_label_text 0" \
		-textvar panvar(data_label_text) -width $textLen 
	pack $nfr.gleft $nfr.gright -side left -padx 10

	set nfr $f.axisLabels
	frame $nfr
	pack $nfr -anchor nw -padx $lPad
	command_entry $nfr.dleft  "X Axis Label: " 15 \
		"bind_textbox $pid x_label_text 0" \
		-textvar panvar(x_label_text) -width $textLen 
	command_entry $nfr.dright  "Y Axis Label: " 15 \
		"bind_textbox $pid y_label_text 0" \
		-textvar panvar(y_label_text) -width $textLen 
	pack $nfr.dleft $nfr.dright -side left -padx 10

	set sWid 8
	label $f.nop2 -text " "
	pack $f.nop2 -side top

	set nfr $f.margins
	frame $nfr
	pack $nfr -anchor nw -padx $padx 
	command_entry $nfr.left  "Left Margin: " 15 \
		"bind_calibrate $pid margin_left" \
		-textvar panvar(margin_left) -width $sWid 
	command_entry $nfr.right  "Right Margin: " 15 \
		"bind_calibrate $pid margin_right" \
		-textvar panvar(margin_right) -width $sWid
	command_entry $nfr.top  "Top Margin: " 15 \
		"bind_calibrate $pid margin_top" \
		-textvar panvar(margin_top) -width $sWid 
	command_entry $nfr.bot  "Bottom Margin: " 15 \
		"bind_calibrate $pid margin_bottom" \
		-textvar panvar(margin_bottom) -width $sWid 
	pack $nfr.left $nfr.right $nfr.top $nfr.bot -side left -padx 10

	frame $f.xb
	pack $f.xb -anchor nw  -padx $padx
	command_entry $f.xb.min "xMin: " 15 \
		"bind_limit $pid x_min" \
		-textvar panvar(x_min) -width $sWid 
	command_entry $f.xb.max " xMax: " 15 \
		"bind_limit $pid x_max" \
		-textvar panvar(x_max) -width $sWid 
    command_entry $f.xb.tics " xTics: " 15 \
		"bind_textbox $pid x_tics 0" \
		-textvar panvar(x_tics) -width $sWid 
    command_entry $f.xb.format " xFormat: " 15 \
		"bind_textbox $pid x_format 0" \
		-textvar panvar(x_format) -width $sWid
	pack $f.xb.min $f.xb.max $f.xb.tics $f.xb.format -side left -padx 10

	frame $f.yb
	pack $f.yb -anchor nw -padx $padx
	command_entry $f.yb.min "yMin: " 15 \
		"bind_limit $pid y_min" \
			-textvar panvar(y_min) -width $sWid 
	command_entry $f.yb.max " yMax: " 15 \
		"bind_limit $pid y_max" \
		-textvar panvar(y_max) -width $sWid 
    command_entry $f.yb.tics  " yTics:" 15 \
		"bind_textbox $pid y_tics 0" \
		-textvar panvar(y_tics) -width $sWid
    command_entry $f.yb.format " yFormat:" 15 \
		"bind_textbox $pid y_format 0" \
		-textvar panvar(y_format) -width $sWid 
	pack $f.yb.min $f.yb.max $f.yb.tics $f.yb.format -side left -padx 10

	label $f.nop3 -text " "
	pack $f.nop3 -side top

	set nfr $f.fit
	frame $nfr
	pack $nfr -padx $padx

	frame $nfr.width
	label $nfr.width.label -text "Fitting Line Width: "
	label $nfr.width.value -textvar panvar(fit_line_width) 
	button $nfr.width.up -text "Wider" -command "bind_fitwidth $pid 1"
	button $nfr.width.down -text "Narrower" -command "bind_fitwidth $pid -1"
	pack $nfr.width.label $nfr.width.value $nfr.width.up $nfr.width.down -side left

	command_entry $nfr.num "Fitting Points: " 18 \
		"bind_fitpoints $pid fit_points $panvar(fit_points)" \
		-textvar panvar(fit_points) -width 4 
	command_entry $nfr.geo "Plot Window Geometry: " 22 \
		"bind_geometry" \
		-textvar panvar($pid,canvas_geometry) -width 18
	pack $nfr.width $nfr.num $nfr.geo -side left -padx 10

	frame $f.but
	pack $f.but
	button $f.but.apply -text "Save Plot" -command "pop_save_plot $pid"
	button $f.but.dismiss -text "Dismiss" -command "destroy $f"
	pack $f.but.apply $f.but.dismiss -side left -pady 10
}

#####################################################################
#
# create a widget which allows for color selection
#
proc set_color {f pid myLabel width cVar item} {
	global xcv panvar

	set padx 10
	frame $f
	label $f.label -text "$myLabel: " -width $width -anchor e
	menubutton $f.color -padx 0 -menu $f.color.menu -width 12 -anchor w \
		-textvar $cVar -underline 0
    menu $f.color.menu
	foreach color $xcv(colors) {
        $f.color.menu add command -label $color -command \
			"bind_setting $pid $item $color 0"
    }
    pack $f.label $f.color -side left
}

#####################################################################
#
# create a widget for changing the background color of a canvas item
#
proc set_background {f pid myLabel cVar canvasItem cVar2 width} {
	global xcv 

	set padx 10
	frame $f
	label $f.label -text "$myLabel: " -width $width -anchor e
	menubutton $f.color -padx 0 -menu $f.color.menu -width 12 -anchor w \
		-textvar $cVar -underline 0
    menu $f.color.menu
	foreach item $xcv(colors) {
        $f.color.menu add command -label $item -command \
			"set $cVar $item; \
			$canvasItem configure -background $item; \
			$canvasItem configure -highlightbackground $item; \
			bind_setting $pid $cVar2 $item 0"
    }
    pack $f.label $f.color -side left
}

#####################################################################
#
# create a widget which gives the user a choice of radio buttons 
#
proc set_radio {f pid myLabel width name item} {
	global xcv vPlot

	set padx 10
	frame $f
	label $f.label -text "$myLabel: " -width $width -anchor e
	pack $f.label -side left
	foreach symbol $xcv(${name}_list) {
		radiobutton $f.$symbol -text $symbol \
			-variable vPlot($pid,$item) \
			-anchor w -value $symbol \
			-command "bind_setting $pid $item $symbol 0"
		pack $f.$symbol -side left
	}
}

#####################################################################
#
# create widgets which allow for font selection
#
proc set_font_attr {pid f myLabel width cVar item} {
	global xcv vPlot panvar

	set padx 10
	frame $f
	label $f.label -text "$myLabel: " -width $width -anchor e
	menubutton $f.font -padx 0 -menu $f.font.menu -width 10 -anchor w \
		-textvar $cVar -underline 0
    pack $f.label $f.font -side left

    menu $f.font.menu
	foreach name $vPlot($pid,${item}_list) {
        $f.font.menu add command -label "$name" \
			-command "bind_font_setting $pid $item $name"
    }
}

#####################################################################
#
# create the string which identifies the font for a plot window 
#
proc bind_font_setting { pid item value } {
	global panvar xcv vPlot

	if {$item == "font_name"} {
		set panvar(font_name) $value
		set style $panvar(font_style)
		set size $panvar(font_size)
		set font "-*-$value-$style-r-normal--$size-*-*-*-m-*-iso8859-1"
	}
	if {$item == "font_style"} {
		set name $panvar(font_name)
		set panvar(font_style) $value
		set size $panvar(font_size)
		set font "-*-$name-$value-r-normal--$size-*-*-*-m-*-iso8859-1"
	}
	if {$item == "font_size"} {
		set name $panvar(font_name)
		set style $panvar(font_style)
		set panvar(font_size) $value
		set font "-*-$name-$style-r-normal--$value-*-*-*-m-*-iso8859-1"
	}
	bind_setting $pid canvas_font $font 0
}

#####################################################################
#####################################################################
#####################################################################

proc bind_textbox { pid item refit } {
	global panvar vPlot xcv

	set newValue $panvar($item)
	bind_setting $pid $item $newValue $refit
}

proc bind_setting { pid item newValue refit } {
	global panvar vPlot xcv

	set vPlot($pid,$item) $newValue
	set panvar($item) $newValue
	if ![info exists vPlot($pid,pNumCur)] {
		write_parms
		return
	}

	set pNum $vPlot($pid,pNumCur)
	set dNum $vPlot($pid,$pNum,dNumCur)

	# make the change for current data only
	if {[string compare $panvar($pid,settings) DATA] == 0} {
		set vPlot($pid,$pNum,$item) $newValue
		set vPlot($pid,$pNum,$dNum,$item) $newValue
		if {$refit > 0} {
			for {set np 0} {$np < $panvar(parms,num)} {incr np} {
				if [info exists vPlot($pid,$pNum,$dNum,parms,$np)] {
					set panvar(parms,$np) $vPlot($pid,$pNum,$dNum,parms,$np)
				}
			}
			calc_fit_one $pid $pNum $dNum
		}
	}

	# make the change for current plot only
	if {[string compare $panvar($pid,settings) PLOT] == 0} {
		set vPlot($pid,$pNum,$item) $newValue
		for {set j 0} {$j < $vPlot($pid,$pNum,dNumTot)} {incr j} {
			set vPlot($pid,$pNum,$j,$item) $newValue
			if {$refit > 0} {
				for {set np 0} {$np < $panvar(parms,num)} {incr np} {
					if [info exists vPlot($pid,$pNum,$j,parms,$np)] {
						set panvar(parms,$np) $vPlot($pid,$pNum,$j,parms,$np)
					}
				}
				calc_fit_one $pid $pNum $j
			}
		}
	}

	# make the change for all plots
	if {[string compare $panvar($pid,settings) ALL] == 0} {
		for {set i 0 } {$i < $vPlot($pid,pNumTot)} {incr i} {
			set vPlot($pid,$i,$item) $newValue
			for {set j 0} {$j < $vPlot($pid,$i,dNumTot)} {incr j} {
				set vPlot($pid,$i,$j,$item) $newValue
				if {$refit > 0} {
					for {set np 0} {$np < $panvar(parms,num)} {incr np} {
						if [info exists vPlot($pid,$i,$j,parms,$np)] {
							set panvar(parms,$np) $vPlot($pid,$i,$j,parms,$np)
						}
					}
					calc_fit_one $pid $i $j
				}
			}
		}
	}

	write_parms
	do_plot $pid 0
}

proc bind_calibrate { pid item } {
	global panvar vPlot xcv

	set newValue $panvar($item)
	# if new value is blank, then put old value back
	if {$newValue == ""} {
		set newValue $vPlot($pid,$item)
		set panvar($item) $newValue
	}

	bind_setting $pid $item $newValue 0
	calibrate_xy $pid

	write_parms
	do_plot $pid 0
}

#####################################################################
#
# limit field binding
#
proc bind_limit { pid item } {
	global vPlot xcv myData panvar

	set newValue $panvar($item)
	if ![info exists myData(0,x_min)] {
		u_error "No input data file."
		return
	}

	if [catch { set x [expr double($panvar($item))] } err] {
		u_error "Do not recognize numeric value in $item, check input field."
		return
	}

	if { $panvar(x_max) <= $panvar(x_min) } {
		u_error "xMin cannot be greater than xMax"
		return
	}

	if { $panvar(y_max) <= $panvar(y_min) } {
		u_error "yMin cannot be greater than yMax"
		return
	}

	set pNum $vPlot($pid,pNumCur)
	bind_setting $pid $item $newValue 1
	calibrate_xy $pid
	calibrate_fit $pid $pNum

	write_parms
	do_plot $pid 0
}

#####################################################################
#
# check fit points entered 
#
proc bind_fitpoints { pid item newValue } {
	global panvar vPlot
 
	if [catch { set iNum [expr int($panvar($item))] } err] {
		u_error "Do not recognize integer value in $item, check input field."
		return
	}

	if { $iNum <= 2 } {
		u_error "Number of fitting points must be > 2."
		return
	}

	if { $iNum >= 1000 } {
		u_error "Number of fitting points must be < 1000."
		return
	}

	set panvar($item) $iNum
	bind_setting $pid $item $iNum 1

	write_parms
	do_plot $pid 0
}

#####################################################################
#
# check fit points entered 
#
proc bind_fitwidth { pid inc } {
	global panvar vPlot

	if { $panvar(fit_line_width) == 1 && $inc < 0 } {
		return
	}

	if { $panvar(fit_line_width) == 9 && $inc > 0 } {
		return
	}

	set panvar(fit_line_width) [expr $panvar(fit_line_width) + $inc]  
	bind_setting $pid fit_line_width $panvar(fit_line_width) 0

	write_parms
	do_plot $pid 0
}

#####################################################################
#
# User manually changes size of the plotting window 
#
proc bind_geometry { } {
	global panvar vPlot xcv
 
 	if ![info exists panvar(plotWinId)] {
		set pid 0
	} else {
		set pid $panvar(plotWinId)
	}
	set newValue $panvar($pid,canvas_geometry)
	set vPlot($pid,canvas_geometry) $newValue

	set pWin .$xcv(win,plot,$pid)
	wm geometry $pWin $vPlot($pid,canvas_geometry)
	calibrate_geometry $pid
	calibrate_xy $pid

	write_parms
	do_plot $pid 0
}
